/*	gmade - GTK MPEG Audio Description Editor
	written by Sidney Berg <sid@oz.net>
	some code borrowed from mpd which written by 
	Woo-jae Jung <jwj95@eve.kaist.ac.kr>	*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>

typedef struct
{
	char songname[30+1];
	char artist[30+1];
	char album [30+1];
	char year[ 4+1];
	char comment [30+1];
	char type;
}ID3;

GtkWidget *entry[6]; // Entry Fields
FILE *fp;
ID3 data;
char genre[128][18];
GtkWidget *genre_label;

static void removespace(char *str,int max)
{
	int i;

	str[max]=0;

	for(i=max-1;i>=0;i--)
		if(((unsigned char)str[i])<26 || str[i]==' ')str[i]=0; else break;
}

static void fillspace(char *str,int max)
{
	int i;

	for(i=0;i<max;i++)
		if(str[i]==0)break;
		for(;i<max;i++)str[i]=' ';
}

static void readID3()
{
	if (parseID3(fp) == 1) {
		fseek(fp,-125,SEEK_END);
	} else {
		fseek(fp,0,SEEK_END);
		data.songname[0] =
		data.artist[0] =
		data.album[0] =
		data.year[0] =
		data.comment[0] =
		data.type = 0;
	}

	fread(data.songname,30,1,fp);removespace(data.songname,30);
	fread(data.artist,30,1,fp);removespace(data.artist,30);
	fread(data.album ,30,1,fp);removespace(data.album ,30);
	fread(data.year, 4,1,fp);removespace(data.year, 4);
	fread(data.comment ,30,1,fp);removespace(data.comment ,30);
	fread(&(data.type) , 1,1,fp);
	g_print("%d", data.type);
}

static void writeID3()
{
	int i;
	char entry_text[18];

	if (parseID3(fp) == 1)
		fseek(fp,-128,SEEK_END);
	else
		fseek(fp,0,SEEK_END);

	for (i = 0; i < 125; i++)
		if (!strcmp(gtk_entry_get_text(GTK_ENTRY(entry[5])), genre[i]))
			data.type = i;

	sprintf(data.songname, "%s", gtk_entry_get_text(GTK_ENTRY(entry[0])));
	sprintf(data.artist, "%s", gtk_entry_get_text(GTK_ENTRY(entry[1])));
	sprintf(data.album, "%s", gtk_entry_get_text(GTK_ENTRY(entry[2])));
	sprintf(data.year, "%s", gtk_entry_get_text(GTK_ENTRY(entry[3])));
	sprintf(data.comment, "%s", gtk_entry_get_text(GTK_ENTRY(entry[4])));

	fprintf(fp,"%c%c%c",0x54,0x41,0x47);
	fillspace(data.songname,30);fwrite(data.songname,30,1,fp);
	fillspace(data.artist,30);fwrite(data.artist,30,1,fp);
	fillspace(data.album ,30);fwrite(data.album ,30,1,fp);
	fillspace(data.year, 4);fwrite(data.year, 4,1,fp);
	fillspace(data.comment ,30);fwrite(data.comment ,30,1,fp);
	fwrite(&(data.type) , 1,1,fp);

}

int parseID3()
{
	fseek(fp,-128,SEEK_END);

	if(getc(fp)==0x54)
		if(getc(fp)==0x41)
			if(getc(fp)==0x47)
			{
				return 1;
			} else {
				return 0;
			}

}

static void updateDisplay()
{
	gtk_entry_set_text(GTK_ENTRY(entry[0]), data.songname);
	gtk_entry_set_text(GTK_ENTRY(entry[1]), data.artist);
	gtk_entry_set_text(GTK_ENTRY(entry[2]), data.album);
	gtk_entry_set_text(GTK_ENTRY(entry[3]), data.year);
	gtk_entry_set_text(GTK_ENTRY(entry[4]), data.comment);
	gtk_entry_set_text(GTK_ENTRY(entry[5]), genre[data.type]);
}

static void destroy_file_selection(GtkWidget *widget, GtkWidget *filew)
{
	gtk_object_destroy(GTK_OBJECT(filew));
}

static void file_ok_signal(GtkWidget *widget, GtkWidget *filew)
{
	if (fp != NULL)
		fclose(fp);

	fp = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew)), "r+");
	destroy_file_selection(NULL, filew);
	if (fp != NULL) {
		readID3();
		updateDisplay();
	} else {
	}
}

static void open_signal(GtkWidget *widget, gpointer gdata)
{
	GtkWidget *filew;

	filew = gtk_file_selection_new("Select a MP3 to Open");

	gtk_signal_connect(GTK_OBJECT(filew), "destroy", (GtkSignalFunc) gtk_widget_destroy, NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->cancel_button), "clicked",
													(GtkSignalFunc) destroy_file_selection, filew);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->ok_button), "clicked",
													(GtkSignalFunc) file_ok_signal, filew);

	gtk_widget_show(filew);
}

static void save_signal(GtkWidget *widget, gpointer gdata)
{
	if (fp != NULL)
		writeID3();
}

static void about_signal(GtkWidget *widget, gpointer gdata)
{
	GtkWidget *dialog, *label, *button;

	dialog = gtk_dialog_new();
	gtk_widget_set_usize(GTK_WIDGET(dialog), 350, 100);
	gtk_window_set_title(GTK_WINDOW(dialog), "gmade: v0.1");

	button = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(destroy_file_selection), dialog);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);	

	label = gtk_label_new("gmade v0.1 - GTK MPEG Audio Description Editor\nWritten by Sidney Berg <sid@oz.net>");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);

	gtk_signal_connect(GTK_OBJECT(dialog), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);
	gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);

	gtk_widget_show(button);
	gtk_widget_show(label);
	gtk_widget_show(dialog);
}

int main(int argc, char *argv[])
{
	GtkWidget *window, *label, *table, *button, *button_box, *genre_box;
	GList *genre_list;
	int i;

	gtk_init(&argc, &argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(window), "gmade: v0.1");
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_exit), NULL);
	gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(gtk_exit), NULL);
	gtk_widget_set_usize(GTK_WIDGET(window), 350, 170);

	table = gtk_table_new(6, 3, FALSE);

/* Create labels */
	label = gtk_label_new("Song"); gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1); gtk_widget_show(label);
	label = gtk_label_new("Artist"); gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2); gtk_widget_show(label);
	label = gtk_label_new("Album"); gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3); gtk_widget_show(label);
	label = gtk_label_new("Year"); gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4); gtk_widget_show(label);
	label = gtk_label_new("Comment");gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5); gtk_widget_show(label);
	label = gtk_label_new("Genre"); gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 5, 6); gtk_widget_show(label);
/* End creation of labels */

/* Create genre list */
	strcpy(genre[0], "Blues");
	strcpy(genre[1], "Classic Rock");
	strcpy(genre[2], "Country");
	strcpy(genre[3], "Dance");
	strcpy(genre[4], "Disco");
	strcpy(genre[5], "Funk");
	strcpy(genre[6], "Grunge");
	strcpy(genre[7], "Hip-Hop");
	strcpy(genre[8], "Jazz");
	strcpy(genre[9], "Metal");
	strcpy(genre[10], "New Age");
	strcpy(genre[11], "Oldies");
	strcpy(genre[12], "Other");
	strcpy(genre[13], "Pop");
	strcpy(genre[14], "R&B");
	strcpy(genre[15], "Rap");
	strcpy(genre[16], "Reggae");
	strcpy(genre[17], "Rock");
	strcpy(genre[18], "Techno");
	strcpy(genre[19], "Industrial");
	strcpy(genre[20], "Alternative");
	strcpy(genre[21], "Ska");
	strcpy(genre[22], "Death Metal");
	strcpy(genre[23], "Pranks");
	strcpy(genre[24], "Soundtrack");
	strcpy(genre[25], "Euro-Techno");
	strcpy(genre[26], "Ambient");
	strcpy(genre[27], "Trip-Hop");
	strcpy(genre[28], "Vocal");
	strcpy(genre[29], "Jazz+Funk");
	strcpy(genre[30], "Fusion");
	strcpy(genre[31], "Trance");
	strcpy(genre[32], "Classical");
	strcpy(genre[33], "Instrumental");
	strcpy(genre[34], "Acid");
	strcpy(genre[35], "House");
	strcpy(genre[36], "Game");
	strcpy(genre[37], "Sound Clip");
	strcpy(genre[38], "Gospel");
	strcpy(genre[39], "Noise");
	strcpy(genre[40], "AlternRock");
	strcpy(genre[41], "Bass");
	strcpy(genre[42], "Soul");
	strcpy(genre[43], "Punk");
	strcpy(genre[44], "Space");
	strcpy(genre[45], "Meditative");
	strcpy(genre[46], "Instrumental Pop");
	strcpy(genre[47], "Instrumental Rock");
	strcpy(genre[48], "Ethnic");
	strcpy(genre[49], "Gothic");
	strcpy(genre[50], "Darkwave");
	strcpy(genre[51], "Techno-Industrial");
	strcpy(genre[52], "Electronic");
	strcpy(genre[53], "Pop-Folk");
	strcpy(genre[54], "Eurodance");
	strcpy(genre[55], "Dream");
	strcpy(genre[56], "Southern Rock");
	strcpy(genre[57], "Comedy");
	strcpy(genre[58], "Cult");
	strcpy(genre[59], "Gangsta");
	strcpy(genre[60], "Top 40");
	strcpy(genre[61], "Christian Rap");
	strcpy(genre[62], "Pop/Funk");
	strcpy(genre[63], "Jungle");
	strcpy(genre[64], "Native American");
	strcpy(genre[65], "Cabaret");
	strcpy(genre[66], "New Wave");
	strcpy(genre[67], "Psychadelic");
	strcpy(genre[68], "Rave");
	strcpy(genre[69], "Showtunes");
	strcpy(genre[70], "Trailer");
	strcpy(genre[71], "Lo-Fi");
	strcpy(genre[72], "Tribal");
	strcpy(genre[73], "Acid Punk");
	strcpy(genre[74], "Acid Jazz");
	strcpy(genre[75], "Polka");
	strcpy(genre[76], "Retro");
	strcpy(genre[77], "Musical");
	strcpy(genre[78], "Rock & Roll");
	strcpy(genre[79], "Hard Rock");
	strcpy(genre[80], "Folk");
	strcpy(genre[81], "Folk/Rock");
	strcpy(genre[82], "National Folk");
	strcpy(genre[83], "Swing");
	strcpy(genre[84], "Bebob");
	strcpy(genre[85], "Latin");
	strcpy(genre[86], "Revival");
	strcpy(genre[87], "Celtic");
	strcpy(genre[88], "Bluegrass");
	strcpy(genre[89], "Avantgarde");
	strcpy(genre[90], "Gothic Rock");
	strcpy(genre[91], "Progressive Rock");
	strcpy(genre[92], "Psychedelic Rock");
	strcpy(genre[93], "Symphonic Rock");
	strcpy(genre[94], "Slow Rock");
	strcpy(genre[95], "Big Band");
	strcpy(genre[96], "Chorus");
	strcpy(genre[97], "Easy Listening");
	strcpy(genre[98], "Acoustic");
	strcpy(genre[99], "Humour");
	strcpy(genre[100], "Speech");
	strcpy(genre[101], "Chanson");
	strcpy(genre[102], "Opera");
	strcpy(genre[103], "Chamber Music");
	strcpy(genre[104], "Sonata");
	strcpy(genre[105], "Symphony");
	strcpy(genre[106], "Booty Bass");
	strcpy(genre[107], "Primus");
	strcpy(genre[108], "Porn Groove");
	strcpy(genre[109], "Satire");
	strcpy(genre[110], "Slow Jam");
	strcpy(genre[111], "Club");
	strcpy(genre[112], "Tango");
	strcpy(genre[113], "Samba");
	strcpy(genre[114], "Folklore");

	genre_list = NULL;
	for (i = 0; i < 115; i++)
		genre_list = g_list_append(genre_list, genre[i]);

	genre_box = gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(genre_box), genre_list);
	gtk_combo_set_value_in_list(GTK_COMBO(genre_box), 1, 1);

	entry[5] = gtk_entry_new();
	entry[5] = GTK_COMBO(genre_box)->entry;

	gtk_table_attach_defaults(GTK_TABLE(table), genre_box, 1, 2, 5, 6);
	gtk_widget_show(genre_box);
/* End genre list creation */

/* Create and connect the the Open, Save, and Exit buttons */
	button_box = gtk_vbox_new(TRUE, 0);

	button = gtk_button_new_with_label("Open");
	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 10);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(open_signal), NULL);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Save");
	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 10);
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_signal), NULL);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("Exit");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(gtk_exit), NULL);
	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 10);
	gtk_widget_show(button);

	button = gtk_button_new_with_label("About");
	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(about_signal), NULL);
	gtk_box_pack_start(GTK_BOX(button_box), button, FALSE, TRUE, 10);
	gtk_widget_show(button);

	gtk_table_attach(GTK_TABLE(table), button_box, 2, 3, 0, 6, GTK_FILL, GTK_FILL, 10, 0);
	gtk_widget_show(button_box);
/* End creation of buttons */

/* Create entry fields */
	entry[0] = gtk_entry_new_with_max_length(30);
	gtk_table_attach_defaults(GTK_TABLE(table), entry[0], 1, 2, 0, 1);
	gtk_widget_show(entry[0]);

	entry[1] = gtk_entry_new_with_max_length(30);
	gtk_table_attach_defaults(GTK_TABLE(table), entry[1], 1, 2, 1, 2);
	gtk_widget_show(entry[1]);

	entry[2] = gtk_entry_new_with_max_length(30);
	gtk_table_attach_defaults(GTK_TABLE(table), entry[2], 1, 2, 2, 3);
	gtk_widget_show(entry[2]);

	entry[3] = gtk_entry_new_with_max_length(4);
	gtk_table_attach_defaults(GTK_TABLE(table), entry[3], 1, 2, 3, 4);
	gtk_widget_show(entry[3]);

	entry[4] = gtk_entry_new_with_max_length(30);
	gtk_table_attach_defaults(GTK_TABLE(table), entry[4], 1, 2, 4, 5);
	gtk_widget_show(entry[4]);
/* End creation of entry fields */

	gtk_container_add(GTK_CONTAINER(window), table);
	gtk_widget_show(table);
	gtk_widget_show(window);

	gtk_main();
}
